import os
from pydoc import pager
from .config import DOCS_PATH


def get_filenames(path):
  """Read the filenames in a dir and yield them.

  Arguments
    path  -- the path to read filenames from

  Returns
    filenames -- list of filename strings (without hidden files)
  """
  for f in os.listdir(path):
    if not f.startswith('.'):
      yield f


def read_docs_file(filepath):
  """Read a docs file.

  Arguments
    filepath -- path to the docs file to read

  Returns
    filestring -- string read from the file
  """
  with open(filepath, encoding='utf-8') as docs_file:
    return docs_file.read()


def get_filename_from_filepath(filepath):
  """Parse the filename (without extension) from a filepath.

  Arguments:
    filepath -- filepath string

  Returns:
    filename -- name of the file in the path (without extension)
  """
  basename = os.path.basename(filepath)
  return os.path.splitext(basename)[0]


def print_available_resources(id2filepath, category):
  """Print the available resources for that category.

  Arguments
    id2filepath -- mapping resource identifiers to their filepaths
    category -- the resource category
  """
  print("Available {}:".format(category))
  print(sorted(list(id2filepath.keys())))


def print_minpage(resource, id2filepath, category):
  """Print the minpage for the requested resource if available.

  Arguments
    id2filepath -- mapping resource identifiers to their filepaths
    resource -- the requested resource (e.g.a bash command)
    category -- the resource category
  """
  try:
    pager(read_docs_file(id2filepath[resource]))
  except KeyError:
    print("No entry for resource '{}' among the available {}."
          .format(resource, category))


def read_resources(category):
  """Read in the available resources for that category.

  Arguments
    category -- the resource category

  Returns
    id2filepath -- mapping resource identifiers to their filepaths
  """
  resources_path = os.path.join(DOCS_PATH, category)
  # list of resource filepaths
  filepaths = [os.path.join(resources_path, filename)
               for filename in get_filenames(resources_path)]
  # map resource identifiers to filepaths
  id2filepath = {get_filename_from_filepath(filepath): filepath
                 for filepath in filepaths}

  return id2filepath



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fh:
  long_description = fh.read()

setup(
  name='miniman',
  version='0.0.3',
  description='minimal manpages.',
  long_description=long_description,
  long_description_content_type='text/markdown',
  url='',
  author='Michael Leue',
  author_email='michael@mleue.com',
  license='',
  classifiers=[
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Documentation',
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
  ],
  entry_points={
    'console_scripts': ['min=min.cli.cli:min_cli'],
  },
  keywords='manpage minimal man',
  packages=find_packages(),
  package_data={'min': ['docs/*/*']},
)



import argparse
from .io_ops import read_resources
from .io_ops import print_available_resources, print_minpage


def min(resource, show_topics):
  """Run min cli.

  Arguments
    resource -- string, identifier of the requested resource
    show_topics -- bool, whether to query for topics instead of bash commands
  """
  category = 'commands'
  if show_topics:
    category = 'topics'

  id2filepath = read_resources(category)

  # if no resource was specified, list available resources
  if not resource:
    print_available_resources(id2filepath, category)
  # otherwise try to find the requested resource and print the minpage
  else:
    print_minpage(resource, id2filepath, category)


def min_cli():
  """CLI entry point."""
  parser = argparse.ArgumentParser()
  parser.add_argument("resource", help="which resource you want a minpage for",
                      nargs='?', default='', type=str)
  parser.add_argument("-t", help="help for topics", action="store_true")
  args = parser.parse_args()
  min(resource=args.resource, show_topics=args.t)
